from setuptools import setup, find_packages

setup(
	name="Bloxflip.py",
	version = "2.2.19",
	author = "TheOnlyIceBear",
	description = "An api wrapper for https://bloxflip.com",
	packages=["bloxflip", "bloxflip/utils"]
)